@@ -1,10 +1,10 @@
 package Spreadsheet::HTML;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 
 use Exporter 'import';
-our @EXPORT_OK = qw( portrait generate landscape tornado flip mirror reverse earthquake tsunami );
+our @EXPORT_OK = qw( generate portrait landscape north east south west );
 
 use Clone;
 use HTML::AutoTag;
@@ -15,28 +15,58 @@ sub portrait    { generate( @_, theta =>   0 ) }
 sub landscape   { generate( @_, theta => -270, tgroups => 0 ) }
 
 sub north   { generate( @_, theta =>    0 ) }
-sub east    { generate( @_, theta =>   90, tgroups => 0 ) }
-sub south   { generate( @_, theta => -180, tgroups => 0 ) }
+sub east    { generate( @_, theta =>   90, tgroups => 0, pinhead => 1 ) }
+sub south   { generate( @_, theta => -180, tgroups => 0, pinhead => 1 ) }
 sub west    { generate( @_, theta => -270, tgroups => 0 ) }
 
-sub generate    {
+sub layout {
+    generate( @_,
+        encodes => '',
+        matrix  => 1,
+        table   => {
+            role => 'presentation',
+            ( map {$_ => 0} qw( border cellspacing cellpadding ) ),
+        },
+        _layout => 1,
+    );
+}
+
+sub generate {
     my %args = _process( @_ );
 
-    $args{theta} *= -1 if $args{flip};
+    $args{theta} *= -1 if $args{theta} and $args{flip};
+
+    if (!$args{theta}) { # north
 
-    if (!$args{theta}) {
         $args{data} = $args{flip} ? [ map [ CORE::reverse @$_ ], @{ $args{data} } ] : $args{data};
+
     } elsif ($args{theta} == -90) {
-        $args{data} = [ map [ CORE::reverse @$_ ], CORE::reverse @{ Math::Matrix::transpose( $args{data} ) }]
-    } elsif ($args{theta} == 90) {
-        $args{data} = [ map [ CORE::reverse @$_ ], @{ Math::Matrix::transpose( $args{data} ) }];
-    } elsif ($args{theta} == -180) {
-        $args{data} = [ CORE::reverse @{ $args{data} } ]
+
+        $args{data} = [ map [ CORE::reverse @$_ ], CORE::reverse @{ Math::Matrix::transpose( $args{data} ) }];
+
+    } elsif ($args{theta} == 90) { # east
+
+        $args{data} = Math::Matrix::transpose( $args{data} );
+        $args{data} = ($args{pinhead} and !$args{headless})
+            ? [ map [ @$_[1 .. $#$_], $_->[0] ], @{ $args{data} } ]
+            : [ map [ CORE::reverse @$_ ], @{ $args{data} } ];
+
+    } elsif ($args{theta} == -180) { # south
+
+        $args{data} = ($args{pinhead} and !$args{headless})
+            ? [ @{ $args{data} }[1 .. $#{ $args{data} }], $args{data}[0] ]
+            : [ CORE::reverse @{ $args{data} } ];
+
     } elsif ($args{theta} == 180) {
+
         $args{data} = [ map [ CORE::reverse @$_ ], CORE::reverse @{ $args{data} } ];
-    } elsif ($args{theta} == -270) {
-        $args{data} = [@{ Math::Matrix::transpose( $args{data} ) }]
+
+    } elsif ($args{theta} == -270) { # west
+
+        $args{data} = [@{ Math::Matrix::transpose( $args{data} ) }];
+
     } elsif ($args{theta} == 270) {
+
         $args{data} = [ CORE::reverse @{ Math::Matrix::transpose( $args{data} ) }];
     }
 
@@ -59,19 +89,10 @@ sub _process {
     my $empty = exists $args->{empty} ? $args->{empty} : '&nbsp;';
     my $max_cols = scalar @{ $data->[0] };
 
-    if ($args->{layout}) {
-        $args->{encodes} = '' unless exists $args->{encodes}; 
-        $args->{matrix} = 1 unless exists $args->{matrix};
-        unless (exists $args->{table}) {
-            $args->{table}{role} = 'presentation';
-            $args->{table}{$_}   = 0 for qw( border cellspacing cellpadding );
-        }
-    }
-
-    # headings is an alias for row0
+    # headings is an alias for -row0
     $args->{-row0} = delete $args->{headings} if exists $args->{headings};
 
-    # headings to index mapping for column
+    # headings to index mapping for -colX
     my %index = ();
     if ($#{ $data->[0] }) {
         %index = map { '-' . $data->[0][$_] || '' => $_ } 0 .. $#{ $data->[0] };
@@ -82,7 +103,7 @@ sub _process {
 
     for my $row (0 .. $#$data) {
 
-        unless ($args->{layout}) {
+        unless ($args->{_layout}) {
             push @{ $data->[$row] }, undef for 1 .. $max_cols - $#{ $data->[$row] } + 1;  # pad
             pop  @{ $data->[$row] } for $max_cols .. $#{ $data->[$row] };                 # truncate
         }
@@ -94,20 +115,26 @@ sub _process {
             # -rowX
             my $attr = $args->{$tag};
             if (exists $args->{"-row$row"}) {
-                if (ref($args->{"-row$row"}) eq 'CODE') {
-                    $val = $args->{"-row$row"}->($val);
-                } elsif (ref($args->{"-row$row"}) eq 'HASH') {
-                    $attr = $args->{"-row$row"};
+                $args->{"-row$row"} = [ $args->{"-row$row"} ] unless ref( $args->{"-row$row"} ) eq 'ARRAY';
+                for (@{ $args->{"-row$row"} }) {
+                    if (ref($_) eq 'CODE') {
+                        $val = $_->($val);
+                    } elsif (ref($_) eq 'HASH') {
+                        $attr = $_;
+                    }
                 }
             }
 
             # -colX
             unless (!$args->{matrix} and $row == 0) {
                 if (exists $args->{"-col$col"}) {
-                    if (ref($args->{"-col$col"}) eq 'CODE') {
-                        $val = $args->{"-col$col"}->($val);
-                    } elsif (ref($args->{"-col$col"}) eq 'HASH') {
-                        $attr = $args->{"-col$col"};
+                    $args->{"-col$col"} = [ $args->{"-col$col"} ] unless ref( $args->{"-col$col"} ) eq 'ARRAY';
+                    for (@{ $args->{"-col$col"} }) {
+                        if (ref($_) eq 'CODE') {
+                            $val = $_->($val);
+                        } elsif (ref($_) eq 'HASH') {
+                            $attr = $_;
+                        }
                     }
                 }
             }
@@ -233,12 +260,12 @@ sub _args {
     return ( $self, Clone::clone($data), $args );
 }
 
-sub transpose   { no warnings; warn "transpose is deprecated, use landscape"; generate( @_, theta => -270, tgroups => 0 ) }
-sub earthquake  { no warnings; warn "earthquake is deprecated, use east"; generate( @_, theta =>  90, tgroups => 0 ) }
-sub reverse     { no warnings; warn "reverse is deprecated, use south with flip"; generate( @_, theta => 180, tgroups => 0 ) }
+sub transpose   { no warnings; warn "transpose is deprecated, use landscape";       generate( @_, theta => -270, tgroups => 0 ) }
+sub earthquake  { no warnings; warn "earthquake is deprecated, use east";           generate( @_, theta =>  90, tgroups => 0 ) }
+sub reverse     { no warnings; warn "reverse is deprecated, use south with flip";   generate( @_, theta => 180, tgroups => 0 ) }
 sub mirror      { no warnings; warn "mirror is deprecated, use portrait with flip"; generate( @_, theta =>    0, flip => 1 ) }
-sub tsunami     { no warnings; warn "tsunami is deprecated, use east with flip"; generate( @_, theta =>  -90, tgroups => 0 ) }
-sub flip        { no warnings; warn "flip is deprecated, use south"; generate( @_, theta => -180, tgroups => 0 ) }
+sub tsunami     { no warnings; warn "tsunami is deprecated, use east with flip";    generate( @_, theta =>  -90, tgroups => 0 ) }
+sub flip        { no warnings; warn "flip is deprecated, use south";                generate( @_, theta => -180, tgroups => 0 ) }
 
 
 1;
@@ -315,6 +342,25 @@ Headers on bottom: C<generate( 'theta', -180 )>
 
 Headers on right: C<generate( 'theta', 90 )>
 
+=item * C<layout( %args )>
+
+Layout tables are not recommended, but if you choose to
+use them you should label them as such. This adds W3C
+recommended layout attributes to the table tag and features:
+emiting only <td> tags, no padding or pruning of rows, forces
+no HTML entity encoding in table cells.
+
+=back
+
+For most cases, C<portrait()> and C<landscape()> are all you need.
+
+=head2 DEPRECATED METHODS
+
+These methods will be removed soon. Using them now will
+emit warning messages that inform of alternatives.
+
+=over 4
+
 =item * C<transpose( %args )>
 
 Deprecated: use C<landscape()>
@@ -341,8 +387,6 @@ Deprecated: use C<south()>
 
 =back
 
-For most cases, C<portrait()> and C<landscape()> are all you need.
-
 =head1 PARAMETERS
 
 All methods/procedures accept the same named parameters.
@@ -355,14 +399,14 @@ array references.
 
 =over 4
 
-=item * C<data: [ [], [], [], ... ]>
+=item * C<data>
 
 The data to be rendered into table cells. Should be
 an array ref of array refs.
 
   data => [["a".."c"],[1..3],[4..6],[7..9]]
 
-=item * C<file: $str>
+=item * C<file>
 
 The name of the data file to read. Supported formats
 are XLS, CSV, JSON, YAML and HTML (first table found).
@@ -380,7 +424,7 @@ Rotates table clockwise. Default to 0: headers at top.
 Flips table horizontally.
 Can be used in conjunction with C<theta>.
 
-=item * C<indent: $str>
+=item * C<indent>
 
 Render the table with nested indentation. Defaults to
 undefined which produces no indentation. Adds newlines
@@ -388,7 +432,7 @@ when set to any value that is defined.
 
   indent => '    '
 
-=item * C<encodes: $str>
+=item * C<encodes>
 
 HTML Encode contents of td tags. Defaults to empty string
 which performs no encoding of entities. Pass a string like
@@ -398,13 +442,22 @@ encoded as HTML entites.
 
   encodes => '<>"'
 
-=item * C<empty: $str>
+=item * C<empty>
 
 Replace empty cells with this value. Defaults to &nbsp;
 Set value to undef to avoid any substitutions.
 
   empty => '&#160;'
 
+=item * C<tgroups: 0 or 1>
+
+Group table rows into <thead> <tfoot> and <tbody>
+sections. The <tfoot> section is always found before
+the <tbody> section. Only available for C<generate()>,
+C<portrait()> and C<mirror()>.
+
+  tgroups => 1
+
 =item * C<cache: 0 or 1>
 
   cache => 1
@@ -417,23 +470,21 @@ Render the table with only td tags, no th tags, if true.
 
   matrix => 1
 
-=item * C<layout: 0 or 1>
-
-Layout tables are not recommended, but if you choose to
-use them you should label them as such. This adds W3C
-recommended layout attributes to the table tag and features:
-emiting only <td> tags, no padding or pruning of rows, forces
-no HTML entity encoding in table cells.
-
-  layout => 1
-
 =item * C<headless: 0 or 1>
 
 Render the table with without the headings row, if true. 
 
   headless => 1
 
-=item * C<headings: \& or \%>
+=item * C<pinhead: 0 or 1>
+
+Works in conjunction with C<theta> to produces tables with
+headings placed on sides other than the top and perserve
+data alignment for reporting readability.
+
+  pinhead => 1
+
+=item * C<headings>
 
 Apply anonymous subroutine to each cell in headings row.
 
@@ -443,7 +494,11 @@ Or apply hash ref as attributes:
 
   headings => { class => 'some-class' }
 
-=item * C<-rowX: \& or \%>
+Or both:
+
+  headings => [ sub { uc shift }, { class => "foo" } ]
+
+=item * C<-rowX>
 
 Apply this anonymous subroutine to row X. (0 index based)
 
@@ -453,7 +508,11 @@ Or apply hash ref as attributes:
 
   -row3 => { class => 'some-class' }
 
-=item * C<-colX: \& or \%>
+Or both:
+
+  -row3 => [ sub { uc shift }, { class => "foo" } ]
+
+=item * C<-colX>
 
 Apply this anonymous subroutine to column X. (0 index based)
 
@@ -461,7 +520,11 @@ Apply this anonymous subroutine to column X. (0 index based)
 
 Or apply hash ref as attributes:
 
-  -colo => { class => 'some-class' }
+  -col4 => { class => 'some-class' }
+
+Or both:
+
+  -col4 => [ sub { uc shift }, { class => "foo" } ]
 
 You can alias any column number by the value of the heading
 name in that column:
@@ -470,16 +533,9 @@ name in that column:
 
   -my_heading3 => { class => 'special-row' }
 
-=item * C<tgroups: 0 or 1>
+  -my_heading3 => [ sub { uc shift }, { class => "foo" } ]
 
-Group table rows into <thead> <tfoot> and <tbody>
-sections. The <tfoot> section is always found before
-the <tbody> section. Only available for C<generate()>,
-C<portrait()> and C<mirror()>.
-
-  tgroups => 1
-
-=item * C<caption: $str or \%>
+=item * C<caption>
 
 Caption is special in that you can either pass a string to
 be used as CDATA or a hash whose only key is the string
@@ -489,7 +545,7 @@ to be used as CDATA:
 
   caption => { "With Attributes" => { align => "bottom" } }
 
-=item * C<colgroup: \@ or \%>
+=item * C<colgroup>
 
 Add colgroup tag(s) to the table. Use an AoH for multiple.
 
@@ -497,7 +553,7 @@ Add colgroup tag(s) to the table. Use an AoH for multiple.
 
   colgroup => [ { span => 20 }, { span => 1, class => 'end' } ]
 
-=item * C<col: \@ or \%>
+=item * C<col>
 
 Add col tag(s) to the table. Use an AoH for multiple. Wraps
 tags within a colgroup tag.
@@ -506,33 +562,33 @@ tags within a colgroup tag.
 
   col => [ { span => 20 }, { span => 1, class => 'end' } ]
 
-=item * C<table: \%>
+=item * C<table>
 
 Apply these attributes to the table tag.
 
   table => { class => 'spreadsheet' }
 
-=item * C<thead: \%>
+=item * C<thead>
 
   thead => { style => 'background: color' }
 
-=item * C<tfoot: \%>
+=item * C<tfoot>
 
   tfoot => { style => 'background: color' }
 
-=item * C<tbody: \%>
+=item * C<tbody>
 
   tbody => { style => 'background: color' }
 
-=item * C<tr: \%>
+=item * C<tr>
 
   tr => { style => { background => [qw( color1 color2 )]' } }
 
-=item * C<th: \%>
+=item * C<th>
 
   th => { style => 'background: color' }
 
-=item * C<td: \%>
+=item * C<td>
 
   td => { style => 'background: color' }
 
